# sb::Admin::Message
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==

# ======================================
package sb::Admin::Message;
use strict;
use sb::Admin::CommentOption ();
use base qw( sb::Admin::CommentOption );
use vars qw( $VERSION );
$VERSION = '0.09';
# 0.09 [2009/03/19] removed br tags from the strings
# 0.08 [2007/11/28] modified to use sb::Utils->remove_duplicates
# 0.07 [2007/11/05] added title_name
# 0.06 [2007/09/26] enabled $self->conf->save
# 0.05 [2007/09/09] ported from Serene Bach 2.*
# 0.04 [2007/04/25] changed _change_message_status to implement adding refusal feature
# 0.03 [2007/02/09] changed _change_message_status to handle 'closed' status
# 0.02 [2005/08/06] changed _clip_for_comment and _display_entry to display list correctly
# 0.01 [2005/07/22] fixed a bug to change status correctly from detail screen
# 0.00 [2005/02/02] generated
# --------------------------------------
# // defines
sub TEMPLATE     (){ 'message.html' };
sub LIST_COLUMN  (){ 7 };
sub DENIED_CHECK (){ '-' };
sub NO_DATA      (){ '-' };
sub DATE_FORMAT  (){ '%YearShort%.%Mon%.%Day% %Hour%:%Min%' };
sub DATE_LANG    (){ 'en' };
# --------------------------------------
# // override base class
sub title_name
{
	my $self = shift;
	my $title = $self->SUPER::title_name();
	return $title if ($title);
	$title = sb->string(' &gt; Comments');
	$title .= sb->string(' &gt; Refusal') if ($self->cgi->value('__type') eq 'refusal');
	return $title;
}
# --------------------------------------
# // public functions
sub callback
{
	my $self = shift;
	if ($self->check_configurability() and $self->cgi->value('__type') eq 'refusal')
	{
		return $self->SUPER::callback(@_)
	}
	$self->{'mid'} = undef;
	return ($self->regi)
		? $self->_update(@_)
		: $self->_display(@_);
}
# --------------------------------------
# // private functions
sub _update
{
	my $self = shift;
	my $msg = undef;
	my @sels = ($self->cgi->value('mid') ne '') ? ($self->cgi->value('mid')) : split("\0",$self->cgi->value('sel'));
	my @coms = sb::Data->load('Message','cond'=>{'id'=>\@sels});
	my @eids = ();
	ACTION_SWITCH: {
		$_ = $self->cgi->value('regi_action');
		/^del$|^refuse$/ && do
		{ # delete
			my $flag = ($_ eq 'refuse');
			my @list = split("\n",$self->conf->value('conf_ip_ban'));
			foreach my $com (@coms)
			{
				push(@list,$com->host) if ($flag);
				push(@eids,$com->eid);
				$com->erase;
			}
			if ($flag)
			{ # register IPs for refusal
				$self->conf->value('conf_ip_ban' => join("\n",sb::Utils->remove_duplicates(@list)));
				$self->conf->save($self->conf->config_file());
			}
			$msg = sprintf(sb->string('Deleted %d item(s).'),$#coms + 1) . "\n";
			last ACTION_SWITCH;
		};
		/^stat(\-1|\d)$/ && do
		{ # change status
			my $new = $1;
			foreach my $com (@coms)
			{
				push(@eids,$com->eid);
				$com->stat($new);
			}
			$msg = sb->string('Modified.') . "\n";
			last ACTION_SWITCH;
		};
	} # end of ACTION_SWITCH
	sb::Data->update(@coms) if (@coms);
	$self->update_entry_attachment(@eids);
	$self->build_list('recent_comment_list');
	return $self->_display('message'=>$msg);
}
sub _display
{
	my $self = shift;
	my %param = (
		'message' => undef,
		'setup'   => $self->setup_list(),
		@_
	);
	if ($self->cgi->value('mid') =~ /^\d+$/ and $self->cgi->value('regi_action') ne 'del')
	{
		$self->{'mid'} = $self->cgi->value('mid');
	}
	my $temp = sb::Template->new($self->load_template('file'=>TEMPLATE));
	my $dispsort = ( $self->cgi->value('dispsort') ne '' ) ? $self->cgi->value('dispsort') : 'date';
	my $page = $self->get_current_page();
	my @coms = sb::Data->load('Message',
		'sort'  => $dispsort,
		'order' => 1,
		'id'    => $self->{'mid'},
		'num'   => $param{'setup'}->{'dispnum'},
		'bgn'   => $page * $param{'setup'}->{'dispnum'},
		'cond'  => $self->_generate_condition(),
	);
	$temp->num(0);
	if ($self->{'mid'} eq '')
	{
		$temp->tag('sb_list_page'=>$self->display_pagelink( # pagelink
				'mode'    => 'comment',
				'column'  => LIST_COLUMN,
				'all'     => sb::Data->matched,
				'printed' => $#coms + 1,
				'num'     => $param{'setup'}->{'dispnum'},
				'params'  => ['dispsort','disptype','dispword','dispdate','dispnum'],
			)
		);
	}
	$self->dispnum_selector( # selector for number
		'cms' => $temp,
		'now' => $param{'setup'}->{'dispnum'},
	);
	$self->monthly_selector( # selector for month
		'cms'  => $temp,
		'tag'  => 'sb_message_dispdate',
		'data' => 'Message',
	);
	foreach my $key ('dispsort','disptype')
	{ # selector for options
		$self->select_option(
			'cms'      => $temp,
			'tag'      => 'sb_' . $key . '_',
			'selected' => ($key eq 'dispsort') ? $dispsort : $self->cgi->value($key),
		);
	}
	$self->listmain(
		'template' => $temp,
		'block'    => 'sb_message_list',
		'objects'  => \@coms,
		'tags'     => {
			'sb_com_id'        => 'id',
			'sb_com_author'    => sub { (shift)->display_message_author(@_); },
			'sb_com_entry'     => \&_display_entry,
			'sb_com_date'      => 'date',
			'sb_com_ip'        => 'host',
			'sb_com_status'    => sub { (shift)->display_message_status(@_) },
			'sb_com_statclass' => 'stat',
			'sb_com_sel'       => \&_display_checkbox,
		},
	);
	if ( $self->{'mid'} ne '' and $coms[0] )
	{
		$temp->num(0);
		my $date = sb::Time->new()->format(
			'time' => $coms[0]->date,
			'form' => DATE_FORMAT,
			'zone' => $coms[0]->tz,
			'lang' => DATE_LANG,
		);
		my $entry = sb::Data->load('Entry','id'=>$coms[0]->eid);
		$temp->tag('sb_com_one_id'     => $coms[0]->id);
		$temp->tag('sb_com_one_author' => $coms[0]->auth ? $coms[0]->auth : NO_DATA);
		$temp->tag('sb_com_one_mail'   => $coms[0]->mail ? $coms[0]->mail : NO_DATA);
		$temp->tag('sb_com_one_url'    => $coms[0]->url  ? $coms[0]->url  : NO_DATA);
		$temp->tag('sb_com_one_body'   => $coms[0]->formated_body);
		$temp->tag('sb_com_one_date'   => $date);
		$temp->tag('sb_com_one_icon'   => $coms[0]->icon_image);
		$temp->tag('sb_com_one_host'   => $coms[0]->host);
		$temp->tag('sb_com_one_entry'  => $self->linked_text(
			'text' => ($entry->subj eq '') ? sb->string('[no title]') : $entry->subj,
			'base' => '?__mode=edit&amp;eid=' . $entry->id,
			'user' => $entry->auth
		));
	}
	$temp->num(0);
	if ($self->cgi->value('disptype') ne '')
	{
		$temp->tag('sb_dispword'=>sb::Text->new($self->cgi->value('dispword'))->entitized());
	}
	$temp->tag('sb_list_selall_status' => 'disabled="disabled"') if ($self->{'mid'} ne '');
	$temp->block('sb_message_typelist'   => int($self->check_configurability()) );
	$temp->block('sb_message_select'     => ($self->{'mid'} eq '') ? 1 : 0);
	$temp->block('sb_message_list_cond'  => ($self->{'mid'} eq '') ? 1 : 0);
	$temp->block('sb_message_one'        => ($self->{'mid'} eq '') ? 0 : 1);
	$temp->block('sb_message_one_status' => ($self->{'mid'} eq '') ? 0 : 1);
	$self->common_parts($temp);
	$self->set_message($temp,$param{'message'});
	return $self->set_main($temp->output)
}
# --------------------------------------
# // private functions - display
sub _generate_condition
{
	my $self = shift;
	my %cond = ();
	if ($self->cgi->value('dispword') ne '' and $self->cgi->value('disptype') ne '')
	{
		$cond{$self->cgi->value('disptype')} = '/' . $self->cgi->value('dispword') . '/';
	}
	if ($self->cgi->value('dispdate') ne '')
	{
		$cond{'date'} = sb::Utils->create_date_condition($self->cgi->value('dispdate'));
		$cond{'__range'} = { 'date' => 'tz' };
	}
	return \%cond;
}
# --------------------------------------
# // private functions - list handlers
sub _display_entry
{
	my $self = shift;
	my $obj  = shift;
	my $entry = sb::Data->load('Entry','id'=>$obj->eid);
	return $self->linked_text(
		'text' => ($entry->subj eq '') ? sb->string('[no title]') : $entry->subj,
		'base' => '?__mode=edit&amp;eid=' . $entry->id . '#comment',
		'user' => $entry->auth,
	);
}
sub _display_checkbox
{
	my $self = shift;
	my $obj  = shift;
	my $entry = sb::Data->load('Entry','id'=>$obj->eid);
	return DENIED_CHECK if ($self->{'mid'} ne '');
	return ( $self->has_permission('id'=>$entry->auth) )
		? '<input type="checkbox" name="sel" value="' . $obj->id . '" class="form-control" />'
		: DENIED_CHECK;
}
1;
__END__
