# sb::App::Feed
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==

# ======================================
package sb::App::Feed;
use strict;
use sb::Data ();
use sb::Text ();
use sb::Content ();
use sb::Content::Feed ();
use sb::App ();
use base qw( sb::App );
use vars qw( $VERSION );
$VERSION = '0.10';
# 0.10 [2008/11/24] modified run to use sb->debug instead of warn
# 0.09 [2008/10/28] debug flag can be enabled by Config
# 0.08 [2007/08/17] moved contents part to sb::Content::Feed
# 0.07 [2007/08/14] modified for Serene Bach 3.*
# 0.06 [2007/04/25] changed _entry to ouput feed_entry_modified correctly
# 0.05 [2005/10/18] changed run to pass TemplateManager object to sb::Content
# 0.04 [2005/08/02] changed run to call sb::Content->output with num
# 0.03 [2005/07/27] changed _entry to convert some fields to avoid an error of parsing xml
# 0.02 [2005/07/11] changed config variable "srv_cgi" to "conf_srv_cgi"
# 0.01 [2005/06/06] changed query to marge into sb.cgi
# 0.00 [2005/05/12] generated
# --------------------------------------
# // defines / static variables
sub DEFAULT_TYPE   (){ 'rss' };
sub CONTENT_TYPE   (){ 'text/xml' };
sub OUTPUT_CHARSET (){ 'utf-8' };
sub OUTPUT_CODE    (){ 'utf8' };
my %mFeedTemplate = (
	'rss'  => 'default_rss.rdf',
	'atom' => 'default_atomfeed.xml',
);
# --------------------------------------
# // override base class
sub initialize
{
	my $self = shift;
	return $self->SUPER::initialize(
		'mode' => 'page',
		'base' => undef,
		'cond' => '',
		'id'   => undef,
		@_
	);
}
sub run
{
	my $class = shift;
	my $self = $class->new(@_);
	$self->_check_mode();
	my $type = $self->cgi->value('feed');
	$type = DEFAULT_TYPE if (! $type or ! defined($mFeedTemplate{$type}) );
	my $content = sb::Content->new('main' => { sb::Content::Feed::_init() });
	$content->debug( $self->conf->value('debug_content') );
	my $output = $self->load_template('file'=>$self->template_name($type));
	if ($output ne '')
	{
		$output = $content->output(sb::Template->new($output),
			'mode' => $self->{'mode'},
			'id'   => $self->{'id'},
			'page' => int( $self->cgi->value('page') ),
			'cond' => $self->{'cond'},
			'time' => $self->time->data(),
			'num'  => $self->conf->value('basic_gen_' . $type),
		);
		if ($self->lang->charcode ne OUTPUT_CODE)
		{
			$output = $self->lang->convert($output,OUTPUT_CODE,$self->lang->charcode);
		}
	}
	sb->debug($content->error()) if ($content->error);
	$self->cgi->head(
		'type'    => CONTENT_TYPE,
		'charset' => OUTPUT_CHARSET,
		'length'  => ($content->debug) ? undef : length($output),
	);
	$self->output($output);
}
# --------------------------------------
# // public functions - class method
sub get_code
{
	my $self = shift;
	return( OUTPUT_CODE );
}
sub template_name
{
	my $self = shift;
	my $type = shift;
	$type = DEFAULT_TYPE if (! defined($mFeedTemplate{$type}) );
	return $mFeedTemplate{$type};
}
# --------------------------------------
# // private functions
sub _check_mode
{
	my $self = shift;
	my ($mode,$id,$cond);
	my $cid = $self->cgi->value('cid');
	$mode = 'ent'  if ($self->cgi->value('eid') =~ /^\d+$/);   # entry
	$mode = 'cat'  if ($cid =~ /^\d+$/);                       # category
	$mode = 'arc'  if ($self->cgi->value('month') =~ /^\d+$/); # monthly archive
	$mode = 'arc'  if ($self->cgi->value('day') =~ /^\d+$/);   # daily archive
	$mode = 'srch' if ($self->cgi->value('search') ne '');     # search
	$mode = 'page' if ($mode eq '');                           # page mode is default
	# checking id
	$id = $self->cgi->value('eid') if ($mode eq 'ent');
	$id = $self->cgi->value('cid') if ($mode eq 'cat');
	# checking filter condition
	$cond = ($self->cgi->value('month') ne '') ? $self->cgi->value('month') : $self->cgi->value('day') if ($mode eq 'arc');
	$cond = $self->cgi->value('search') if ($mode eq 'srch');
	# generate path
	my $path = $self->conf->value('conf_srv_cgi') . $self->conf->value('basic_feed');
	$path .= ($self->cgi->value('feed') ne '') ? $self->cgi->value('feed') : DEFAULT_TYPE;
	$path .= '&amp;eid=' . $id if ($mode eq 'ent');
	$path .= '&amp;cid=' . $id if ($mode eq 'cat');
	$path .= '&amp;month=' . $cond if ($mode eq 'arc' and $self->cgi->value('month') ne '');
	$path .= '&amp;day='   . $cond if ($mode eq 'arc' and $self->cgi->value('day') ne '');
	$path .= '&amp;search=' . _encode_uri($cond) if ($mode eq 'srch');
	$path .= '&amp;page=' . $self->cgi->value('page') if ($self->cgi->value('page') =~ /^\d+$/);
	sb::Content::FeedParam->new->path($path);
	$self->{'mode'} = $mode;
	$self->{'id'} = $id;
	$self->{'cond'} = $cond;
}
sub _encode_uri
{
	my $text = shift;
	$text =~ s/(\W)/'%' . unpack('H2', $1)/eg;
	return($text);
}
1;
__END__
