# sb::Local::jp
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==

# ======================================
package sb::Local::jp;
use strict;
use sb::Local ();
use base qw( sb::Local );
use vars qw( $VERSION );
$VERSION = '0.02';
# 0.02 [2006/03/22] changed timezone to use new API
# 0.01 [2006/03/17] changed bracket rule
# 0.00 [2005/10/25] generated
# --------------------------------------
# // override base class
sub timezone
{
	my $self = shift;
	my $obj  = shift;
	return $self->SUPER::timezone($obj,'Asia/Tokyo');
}
sub holidays
{
	my $self = shift;
	my $year = shift;
	my %list = (
		'0101' => 'New Year\'s Day',
		'0211' => 'Foundation Day',
		'0429' => 'Green Day',
		'0503' => 'Constitution Memorial Day',
		'0505' => 'Children\'s Day',
		'1103' => 'Culture Day',
		'1123' => 'Labor Thanksgiving Day',
		'1223' => 'Emperor\'s Birthday',
	);
	if ($year >= 2007)
	{
		$list{'0429'} = 'Showa Day';
		$list{'0504'} = 'Green Day';
	}
	if ($year < 2000)
	{
		$list{'0115'} = 'Adult\'s Day';
		$list{'1010'} = 'Sports Day';
	}
	else
	{
		my $day;
		$day = $self->_weekday('year'=>$year,'mon'=> 1,'nth'=>2,'week'=>1);
		$list{'01' . $self->_pad0($day)} = 'Adult\'s day';
		$day = $self->_weekday('year'=>$year,'mon'=>10,'nth'=>2,'week'=>1);
		$list{'10' . $self->_pad0($day)} = 'Sports day';
	}
	if ($year < 2003)
	{
		$list{'0720'} = 'Marine Day' if ($year > 1995);
		$list{'0915'} = 'Old People\'s Day';
	}
	else
	{
		my $day;
		$day = $self->_weekday('year'=>$year,'mon'=> 7,'nth'=>3,'week'=>1);
		$list{'07' . $self->_pad0($day)} = 'Marine Day';
		$day = $self->_weekday('year'=>$year,'mon'=> 9,'nth'=>3,'week'=>1);
		$list{'09' . $self->_pad0($day)} = 'Old People\'s Day';
	}
	{ # Vernal/Autumnal equinox // the following code is valid from 1980 to 2099.
		my $spring = int(20.8431 + 0.242194*($year-1980)-int(($year-1980)/4));
		my $autumn = int(23.2488 + 0.242194*($year-1980)-int(($year-1980)/4));
		my $weekday = sb::Time->get_weekday('year'=>$year,'mon'=>9,'day'=>$autumn - 1);
		$list{'03' . $spring} = 'Vernal Equinox';
		$list{'09' . $autumn} = 'Autumnal Equinox';
		$list{'09' . ($autumn-1)} = 'National Hoilday' if ($weekday == 2);
	}
	$list{'0504'} = 'National Hoilday' if (sb::Time->get_weekday('year'=>$year,'mon'=>5,'day'=>4) > 1 and $year < 2007);
	foreach my $date (keys(%list))
	{ # A holiday in lieu
		my $mon = substr($date,0,2);
		my $day = substr($date,2,2);
		my $weekday = sb::Time->get_weekday('year'=>$year,'mon'=>$mon,'day'=>$day);
		$list{$mon . $self->_pad0($day + 1)} = 'A holiday in lieu' if ($weekday == 0);
	}
	return $self->holidays_base(%list);
}
1;
__END__
